import http.client
import json
import ssl
from multiprocessing import Pool


class Deletor:

    def __init__(self, host, api_key, port=None, use_ssl=True, verify=True, connections=32):
        self.host = host
        self.port = port
        self.ssl = use_ssl
        self.verify = verify
        self.connections = connections

        self.api_key = api_key

    def _conn(self):
        if self.ssl:
            if self.verify:
                return http.client.HTTPSConnection(self.host, self.port)
            return http.client.HTTPSConnection(self.host, self.port, context=ssl._create_unverified_context())
        return http.client.HTTPConnection(self.host, self.port)

    def delete_alert(self, alert):
        conn = self._conn()
        conn.request("DELETE", "/api/alert/" + alert['id'], headers={'Authorization': 'Bearer ' + self.api_key})
        conn.getresponse()

    def delete_alerts(self):
        conn = self._conn()
        conn.request("GET", "/api/alert?range=all", headers={'Authorization': 'Bearer ' + self.api_key})
        response = conn.getresponse()
        resp = response.read()

        alerts = json.loads(resp)

        with Pool(processes=self.connections) as pool:
            pool.map(self.delete_alert, alerts)

    def delete_case(self, case):
        conn = self._conn()
        conn.request("DELETE", "/api/case/" + case['id'], headers={'Authorization': 'Bearer ' + self.api_key})
        conn.getresponse()

    def delete_cases(self):
        conn = self._conn()
        conn.request("GET", "/api/case?range=all", headers={'Authorization': 'Bearer ' + self.api_key})
        response = conn.getresponse()
        resp = response.read()

        cases = json.loads(resp)

        with Pool(processes=self.connections) as pool:
            pool.map(self.delete_case, cases)



import datetime
import json
import os
from multiprocessing import Pool

import urllib3
from thehive4py.api import TheHiveApi
from thehive4py.query import Between

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class Backupper:

    def __init__(self, backupdir: str, url: str, api_key: str, org: str = None, verify: bool = True):
        url = url.strip("/")
        self.api = TheHiveApi(url, api_key, organisation=org)

        if not verify:
            self.api.cert = False

        self.backupdir = f'{backupdir}-{int(datetime.datetime.utcnow().timestamp())}'
        os.makedirs(self.backupdir, exist_ok=True)
        self.case_file = os.path.join(self.backupdir, 'cases.jsonl')
        self.alert_file = os.path.join(self.backupdir, 'alerts.jsonl')

    def get_file(self, attachment_id: str):
        os.makedirs(os.path.join(self.backupdir, 'attachments'), exist_ok=True)
        response = self.api.download_attachment(attachment_id)
        with open(os.path.join(self.backupdir, 'attachments', attachment_id), 'wb') as io:
            io.write(response.content)

    def backup_cases_all(self) -> [dict]:
        cases = self.api.find_cases(query={}, sort=['-createdAt'], range='all').json()
        self._backup_cases(cases)

    def backup_cases_range(self, start, end) -> [dict]:
        query = Between("createdAt", start, end)
        cases = self.api.find_cases(query=query, sort=['-createdAt'], range='all').json()
        self._backup_cases(cases)

    def _backup_cases(self, cases: [dict]):
        with open(self.case_file, 'w+', encoding='utf8') as io:
            with Pool(processes=8) as pool:
                for case in cases:
                    json.dump(case, io)
                    io.write('\n')

                pool.map(self._backup_case, cases)
                pool.close()
                pool.join()

    def _backup_case(self, case: dict):
        self.backup_observables(case['id'])
        self.backup_tasks(case['id'])

    def backup_tasks(self, case_id: str) -> [dict]:
        tasks = self.api.get_case_tasks(case_id=case_id).json()
        if tasks:
            case_path = os.path.join(self.backupdir, 'cases', case_id)
            os.makedirs(case_path, exist_ok=True)
            with open(os.path.join(case_path, 'tasks.jsonl'), 'w+', encoding='utf8') as io:
                for task in tasks:
                    json.dump(task, io)
                    io.write('\n')
                    self.backup_logs(case_id, task['id'])

    def backup_logs(self, case_id: str, task_id: str) -> [dict]:
        logs = self.api.get_task_logs(task_id).json()
        if logs:
            task_path = os.path.join(self.backupdir, 'cases', case_id, 'tasks', task_id)
            os.makedirs(task_path, exist_ok=True)
            with open(os.path.join(task_path, 'logs.jsonl'), 'w+', encoding='utf8') as io:
                for log in logs:
                    json.dump(log, io)
                    io.write('\n')
                    if 'attachment' in log:
                        self.get_file(log['attachment']['id'])

    def backup_observables(self, case_id: str):
        observables = self.api.get_case_observables(case_id).json()
        if observables:
            os.makedirs(os.path.join(self.backupdir, 'cases', case_id), exist_ok=True)
            with open(os.path.join(self.backupdir, 'cases', case_id, 'observables.jsonl'), 'w+', encoding='utf8') as io:
                for observable in observables:
                    json.dump(observable, io)
                    io.write('\n')
                    if 'attachment' in observable:
                        self.get_file(observable['attachment']['id'])

    def backup_alerts_all(self):
        alerts = self.api.find_alerts(query={}, sort=['-createdAt'], range='all').json()
        self._backup_alerts(alerts)

    def backup_alerts_range(self, start, end):
        query = Between("createdAt", start, end)
        alerts = self.api.find_alerts(query=query, sort=['-createdAt'], range='all').json()
        self._backup_alerts(alerts)

    def _backup_alerts(self, alerts):
        with open(self.alert_file, 'w+', encoding='utf8') as io:
            for alert in alerts:
                json.dump(alert, io)
                io.write('\n')



import argparse
import datetime
import logging
import sys
import time

import urllib3

from thehivebackup.backup import Backupper
from thehivebackup.empty import Deletor
from thehivebackup.migrate3to4 import migrate
from thehivebackup.restore import Restorer

logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser(prog="thehivebackup", description="Backup and restore TheHive remotely.")
    parser.add_argument("-v", "--verbose", help="Show logging.", dest='verbose', action="store_true", default=False)

    subparsers = parser.add_subparsers(title="subcommands", dest="subcommand", required=True)

    parser_backup = subparsers.add_parser('backup', description='Backup TheHive')
    parser_backup.add_argument('--key', type=str, required=True, help='TheHive api key')
    parser_backup.add_argument('--org', type=str, required=False, help='Organisation')
    parser_backup.add_argument('--year', type=int, help='Pass --year and --month to backup a single month')
    parser_backup.add_argument('--month', type=int, help='Pass --year and --month to backup a single month')
    parser_backup.add_argument('--day', type=int, help='Pass --year and --month to backup a single month')
    parser_backup.add_argument("--no-verify", help="Set to false to disable ssl verification", dest='verify',
                               action="store_false", default=True)
    parser_backup.add_argument('host', type=str, help='host of TheHive api')

    parser_migrate = subparsers.add_parser('migrate3to4', description='Migrate TheHive')
    parser_migrate.add_argument('--usermapping', type=str, help='User mapping csv file')
    parser_migrate.add_argument('--fieldmapping', type=str, help='Custom field mapping csv file')
    parser_migrate.add_argument('--metricmapping', type=str, help='Metric mapping csv file')
    parser_migrate.add_argument('--add_old_no', action="store_true", default=False,
                                help='Add old case number as "old-case-no" and "old-case-id"')
    parser_migrate.add_argument('--default-user', type=str, required=True, help='default user')
    parser_migrate.add_argument('backup', type=str, help='TheHive backup')

    parser_restore = subparsers.add_parser('restore', description='Restore TheHive')
    parser_restore.add_argument('--key', type=str, required=True, help='TheHive api key')
    parser_restore.add_argument('--org', type=str, required=False, help='Organisation')
    parser_restore.add_argument('--connections', type=int, help='maximum connections')
    parser_restore.add_argument('--alerts', action="store_true", default=False, help='restore alerts')
    parser_restore.add_argument("--no-verify", help="Set to false to disable ssl verification", dest='verify',
                                action="store_false", default=True)
    parser_restore.add_argument('backup', type=str, help='TheHive backup')
    parser_restore.add_argument('host', type=str, help='host of TheHive api')

    parser_clear = subparsers.add_parser('clear', description='Clean TheHive')
    parser_clear.add_argument('--key', type=str, required=True, help='TheHive api key')
    parser_clear.add_argument('--org', type=str, required=False, help='Organisation')
    parser_clear.add_argument('--connections', type=int, help='maximum connections')
    parser_clear.add_argument("--no-verify", help="Set to false to disable ssl verification", dest='verify',
                              action="store_false", default=True)
    parser_clear.add_argument('host', type=str, help='host of TheHive api')

    if len(sys.argv) < 2:
        parser.print_usage()
        sys.exit(1)

    args = parser.parse_args()

    if args.verbose:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    if args.subcommand == "backup":
        backup(args)
    elif args.subcommand == "migrate3to4":
        migrate(args)
    elif args.subcommand == "restore":
        restore(args)
    elif args.subcommand == "clear":
        clear(args)


def parse_url(url_str: str) -> (bool, str, int):
    url = urllib3.util.parse_url(url_str)
    ssl = True
    if url.scheme is not None and url.scheme == "http":
        ssl = False
    if url.host is None:
        print("Host invalid.")
        sys.exit(1)
    port = url.port
    if port is None:
        port = 443
    return ssl, url.host, port


def clear(args):
    ssl, host, port = parse_url(args.host)
    deletor = Deletor(host, args.key, port, ssl, args.verify, args.connections)
    deletor.delete_cases()
    deletor.delete_alerts()


def restore(args):
    start_time = time.time()
    ssl, host, port = parse_url(args.host)
    print(f'Start at {datetime.datetime.now().isoformat()}')
    restorer = Restorer(args.backup, host, port, args.key, args.connections, ssl, args.verify)
    restorer.store_cases()
    if args.alerts:
        restorer.restore_alerts()
    print(f'Restore done in {time.time() - start_time} seconds')


def utc_nano_timestamp(year: int, month: int, day: int) -> int:
    return int(datetime.datetime(year, month, day, tzinfo=datetime.timezone.utc).timestamp() * 1000)


def backup(args):
    start_time = time.time()
    name = "backup"
    if args.org:
        name += f"-{args.org}"
    if args.year is None and args.month is None:
        migration = Backupper(name + "-full", args.host, args.key, args.org, args.verify)
        migration.backup_cases_all()
        migration.backup_alerts_all()
    elif args.year is not None and args.month is not None:
        if args.day is None:
            name += f"-{args.year}-{args.month}"
            start = utc_nano_timestamp(args.year, args.month, 1)
            next_month = args.month + 1
            if next_month == 13:
                next_month = 1
            end = utc_nano_timestamp(args.year, next_month, 1) - 1
        else:
            name += f"-{args.year}-{args.month}-{args.day}"
            start = utc_nano_timestamp(args.year, args.month, args.day)
            end = start + 60 * 60 * 24 * 1000 - 1
        migration = Backupper(name, args.host, args.key, args.org, args.verify)
        migration.backup_cases_range(start, end)
        migration.backup_alerts_range(start, end)
    else:
        print("Either --year and --month need to be given or none of both.")
        sys.exit(1)
    print(f'Backup done in {time.time() - start_time:.2f} seconds')


if __name__ == '__main__':
    main()



import datetime
import json
import logging
import os
from multiprocessing import Pool

import requests
from urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)  # pylint: disable=no-member


class Restorer:

    def __init__(self, backupdir: str, host: str, port: int, api_key: str, conn: int = 32, ssl=True, verify=True):
        self.host = host
        self.port = port
        self.ssl = ssl
        self.verify = verify
        self.api_key = api_key
        self.connections = conn

        self.backupdir = backupdir
        self.case_file = os.path.join(backupdir, 'cases.jsonl')
        self.alert_file = os.path.join(backupdir, 'alerts.jsonl')

    def request(self, method: str, url: str, api_key: str, data: dict, files: dict = None) -> requests.Response:
        headers = {'Authorization': 'Bearer ' + api_key}
        url_str = f"http://{self.host}:{self.port}{url}"
        if self.ssl:
            url_str = f"https://{self.host}:{self.port}{url}"
        if files is None:
            response = requests.request(method, url_str, headers=headers, verify=self.verify, json=data)
        else:
            data = {"_json": json.dumps(data)}
            response = requests.request(method, url_str, headers=headers, verify=self.verify, data=data, files=files)
        if response.status_code != 200 and response.status_code != 201:
            if "id" in data:
                logging.warning("id: %s", data['id'])
            logging.warning(url_str)
            logging.warning(data)
            logging.warning(response.text)
        return response

    def restore_file(self, url: str, api_key: str, case_object: dict) -> requests.Response:
        filename = case_object['attachment']['name']
        file_id = case_object['attachment']['id']
        with open(os.path.join(self.backupdir, 'attachments', file_id), 'rb') as io:
            file_data = io.read()
        del case_object['attachment']
        return self.request('POST', url, api_key, case_object, {'attachment': (filename, file_data)})

    def store_cases(self):
        with open(self.case_file, 'r') as io:
            with Pool(processes=self.connections) as pool:
                pool.map(self.restore_case_line, io)

    def restore_case_line(self, line):
        case = json.loads(line)
        old_case_id = case['id']
        response = self.request('POST', '/api/case', self.api_key, case)
        if response.status_code == 200 or response.status_code == 201:
            # new_case_id = json.loads(response.read())['id']
            new_case_id = response.json()['id']

            if 'owner' in case and case['owner']:
                self.request('PATCH', f'/api/case/{new_case_id}', self.api_key, {'owner': case['owner']})

            if os.path.exists(os.path.join(self.backupdir, 'cases', old_case_id)):
                self.restore_observables(old_case_id, new_case_id)
                self.restore_tasks(old_case_id, new_case_id)

    def restore_observables(self, old_case_id: str, new_case_id: str):
        observables_path = os.path.join(self.backupdir, 'cases', old_case_id, 'observables.jsonl')
        if os.path.exists(observables_path):
            with open(observables_path) as io:
                for line in io:
                    observable = json.loads(line)
                    if 'attachment' in observable:
                        self.restore_file(f'/api/case/{new_case_id}/artifact', self.api_key, observable)
                    else:
                        self.request('POST', f'/api/case/{new_case_id}/artifact', self.api_key, observable)

    def restore_tasks(self, old_case_id: str, new_case_id: str):
        task_path = os.path.join(self.backupdir, 'cases', old_case_id, 'tasks.jsonl')
        if os.path.exists(task_path):
            with open(task_path) as io:
                for line in io:
                    task = json.loads(line)
                    response = self.request('POST', f'/api/case/{new_case_id}/task', self.api_key, task)

                    if response.status_code == 200 or response.status_code == 201:
                        task_id = response.json()['id']

                        if 'owner' in task and task['owner']:
                            self.request('PATCH', f'/api/case/task/{task_id}', self.api_key, {'owner': task['owner']})

                        if os.path.exists(os.path.join(self.backupdir, 'cases', old_case_id, 'tasks', task['id'])):
                            self.restore_logs(old_case_id, task['id'], task_id)

    def restore_logs(self, old_case_id: str, old_task_id: str, new_task_id: str):
        logs_path = os.path.join(self.backupdir, 'cases', old_case_id, 'tasks', old_task_id, 'logs.jsonl')
        if os.path.exists(logs_path):
            with open(logs_path) as io:
                for line in io:
                    log = json.loads(line)
                    timestamp = datetime.datetime.fromtimestamp(log['createdAt'] / 1000).isoformat()
                    log['message'] = f'Original log by: {log["owner"]} Created at: {timestamp}\n\n{log["message"]}'
                    if 'attachment' in log:
                        response = self.restore_file(f'/api/case/task/{new_task_id}/log', self.api_key, log)
                    else:
                        response = self.request('POST', f'/api/case/task/{new_task_id}/log', self.api_key, log)
                    if response.status_code == 200 or response.status_code == 201:
                        log_id = response.json()['id']

                        if 'owner' in log and log['owner']:
                            self.request('PATCH', f'/api/case/task/log/{log_id}', self.api_key, {'owner': log['owner']})

    def restore_alerts(self):
        with open(self.alert_file, 'r') as io, Pool(processes=self.connections) as pool:
            pool.map(self.restore_alert_line, io)

    def restore_alert_line(self, line):
        self.request('POST', '/api/alert/', self.api_key, json.loads(line))



import csv
import json
import os
import shutil


def ensure_custom_fields(case):
    if "customFields" not in case:
        case["customFields"] = {}
    return case


def add_old_no(case):
    case["customFields"]['old-case-no'] = case['caseId']
    case["customFields"]['old-case-id'] = case['id']
    return case


def migrate_fields(case: dict, field_mapping_file: str) -> dict:
    with open(field_mapping_file, mode='r') as io:
        reader = csv.reader(io)
        field_mapping = dict(reader)

    new_fields = {}
    for field in case["customFields"]:
        if field in field_mapping:
            new_fields[field_mapping[field]] = case["customFields"][field]
    case["customFields"] = new_fields

    return case


def migrate_metrics(case: dict, metrics_mapping_file: str) -> dict:
    with open(metrics_mapping_file, mode='r') as io:
        reader = csv.reader(io)
        metric_mapping = dict(reader)

    new_fields = {}
    if "metrics" in case:
        for metric in case["metrics"]:
            if metric in metric_mapping:
                new_fields[metric_mapping[metric]] = {"integer": case["metrics"][metric]}
            if metrics_mapping_file == "":
                new_fields[metric] = {"integer": case["metrics"][metric]}
        case["customFields"].update(new_fields)
        del case["metrics"]

    return case


def migrate_user(case: dict, user_mapping_file: str, default_user: str) -> dict:
    users = {}
    if user_mapping_file:
        with open(user_mapping_file, mode='r') as io:
            reader = csv.reader(io)
            users = dict(reader)
    if 'owner' in case:
        if case['owner'] in users:
            case['owner'] = users[case['owner']]
        else:
            case['owner'] = default_user
    return case


def replace_user(filename, usermapping, default_user: str):
    if os.path.exists(filename):
        with open(filename) as io, open(filename + '.cpy', 'w+') as writer:
            for line in io:
                task = json.loads(line)
                task = migrate_user(task, usermapping, default_user)
                json.dump(task, writer)
                writer.write('\n')
        shutil.move(filename, filename + '.bak')
        shutil.move(filename + '.cpy', filename)


def migrate(args):
    with open(os.path.join(args.backup, 'cases.jsonl')) as io, \
            open(os.path.join(args.backup, 'cases.jsonl.cpy'), 'w+') as writer:
        for line in io:
            case = json.loads(line)
            case = ensure_custom_fields(case)
            if args.add_old_no:
                case = add_old_no(case)
            if args.fieldmapping:
                case = migrate_fields(case, args.fieldmapping)
            case = migrate_metrics(case, args.metricmapping)
            case = migrate_user(case, args.usermapping, args.default_user)

            json.dump(case, writer)
            writer.write('\n')

    shutil.move(os.path.join(args.backup, 'cases.jsonl'), os.path.join(args.backup, 'cases.jsonl.bak'))
    shutil.move(os.path.join(args.backup, 'cases.jsonl.cpy'), os.path.join(args.backup, 'cases.jsonl'))

    users = args.usermapping
    for casefolder in os.listdir(os.path.join(args.backup, 'cases')):
        replace_user(os.path.join(args.backup, 'cases', casefolder, 'tasks.jsonl'), users, args.default_user)
        replace_user(os.path.join(args.backup, 'cases', casefolder, 'observables.jsonl'), users, args.default_user)

        taskdir = os.path.join(args.backup, 'cases', casefolder, "tasks")
        if os.path.exists(taskdir):
            for taskfolder in os.listdir(taskdir):
                replace_user(os.path.join(taskdir, taskfolder, 'logs.jsonl'), users, args.default_user)



from setuptools import setup

setup(
    name='thehivebackup',
    version='0.1.3',
    url='https://github.com/IFX-CDC/thehivebackup',
    author='Anton Piiroja & Jonas Plum',
    description='Backup and restore TheHive remotely',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=['thehivebackup'],
    install_requires=[
        'requests==2.25.1',
        'thehive4py==1.8.1',
    ],
    entry_points={
        'console_scripts':
            ['thehivebackup = thehivebackup.main:main']
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    zip_safe=False,
)
